import 'package:grizzly/grizzly.dart';

extension StringIterableExt on Iterable<String?> {
  Iterable<bool?> get areAlphaNumeric =>
      map((e) => e?.startsWith(RegExp(r'[a-zA-Z0-9]*$')));

  bool get areAllAlphaNumeric => areAlphaNumeric.isTrue;

  Iterable<bool?> get areAlphabet =>
      map((e) => e?.startsWith(RegExp(r'[a-zA-Z]*$')));

  bool get areAllAlphabet => areAlphabet.isTrue;

  Iterable<bool?> get areDecimal =>
      map((e) => e?.startsWith(RegExp(r'\d+(\.\d+)?$')));

  bool get areAllDecimal => areDecimal.isTrue;

  Iterable<bool?> get areInteger =>
      map((e) => e?.startsWith(RegExp(r'[+-]?\d+$')));

  Iterable<bool?> get areLowerCase =>
      map((e) => e?.startsWith(RegExp(r'[a-z]*$')));

  bool get areAllLowerCase => areLowerCase.isTrue;

  Iterable<bool?> get areUpperCase =>
      map((e) => e?.startsWith(RegExp(r'[A-Z]*$')));

  bool get areAllUpperCase => areUpperCase.isTrue;

  Iterable<bool?> get areSpace => map((e) => e?.startsWith(RegExp(r'\s*$')));

  bool get areAllSpace => areSpace.isTrue;

  Iterable<bool?> areStartsWith(Pattern pattern, [int index = 0]) =>
      map((e) => e?.startsWith(pattern, index));

  bool areAllStartsWith(Pattern pattern, [int index = 0]) =>
      areStartsWith(pattern, index).isTrue;

  Iterable<bool?> areEndsWith(String pattern, [int index = 0]) =>
      map((e) => e?.endsWith(pattern));

  bool areAllEndsWith(String pattern, [int index = 0]) =>
      areEndsWith(pattern, index).isTrue;

  Iterable<int?> get lengths => map((e) => e?.length);

  Iterable<bool?> get areEmpty => map((e) => e?.isEmpty);

  bool get areAllEmpty => areEmpty.isTrue;

  Iterable<bool?> get areNotEmpty => map((e) => e?.isNotEmpty);

  bool get areAllNotEmpty => areNotEmpty.isTrue;

  Iterable<int?> get toInts => map((e) => e == null ? null : int.parse(e));

  Iterable<double?> get toDoubles =>
      map((e) => e == null ? null : double.parse(e));
}
