import 'package:grizzly/src/utils/norepeat_sampler.dart';

Iterable<R> comap<R, T1, T2>(
    Iterable<T1> it1, Iterable<T2> it2, R Function(T1? a, T2? b) mapper,
    {bool fail = false}) sync* {
  Iterator<T1> iter1 = it1.iterator;
  Iterator<T2> iter2 = it2.iterator;

  while (true) {
    final has1 = iter1.moveNext();
    final has2 = iter2.moveNext();

    if (!has1 && !has2) {
      break;
    }

    if (!has1 || !has2) {
      if (fail) {
        throw LengthMismatch();
      }
    }

    yield mapper(has1 ? iter1.current : null, has2 ? iter2.current : null);
  }
}

Iterable<R> comapEq<R, T1, T2>(
    Iterable<T1> it1, Iterable<T2> it2, R Function(T1 a, T2 b) mapper) sync* {
  Iterator<T1> iter1 = it1.iterator;
  Iterator<T2> iter2 = it2.iterator;

  while (iter1.moveNext() && iter2.moveNext()) {
    yield mapper(iter1.current!, iter2.current!);
  }
}

R cofold<R, T1, T2>(R start, Iterable<T1> t1, Iterable<T2> t2,
    R Function(R currentValue, T1? a, T2? b) folder,
    {bool fail = false}) {
  Iterator<T1> iter1 = t1.iterator;
  Iterator<T2> iter2 = t2.iterator;

  R ret = start;

  while (true) {
    final has1 = iter1.moveNext();
    final has2 = iter2.moveNext();

    if (!has1 && !has2) {
      break;
    }

    if (!has1 || !has2) {
      if (fail) {
        throw LengthMismatch();
      }
    }

    ret = folder(ret, has1 ? iter1.current : null, has2 ? iter2.current : null);
  }

  return ret;
}
