import 'package:grizzly/grizzly.dart';
import 'package:test/expect.dart';
import 'package:test/scaffolding.dart';

void main() {
  group('IndexedList', () {
    test('indexOf', () {
      final list = IndexedList(
          ['Mars', 'Alchemist', 'Pangolier', 'Snapfire', 'Oracle', 'Mars']);
      expect(list.indexOf('Mars'), 0);
      expect(list.indexOf('Tide hunter'), -1);
      expect(list.indexOf('Mars', 1), 5);
    });
    test('contains', () {
      final list = IndexedList(
          ['Mars', 'Alchemist', 'Pangolier', 'Snapfire', 'Oracle', 'Tiny']);
      expect(list.contains('Mars'), true);
      expect(list.contains('Pangolier'), true);
      expect(list.contains('Tiny'), true);
      expect(list.contains('Tide hunter'), false);
    });
    test('add', () {
      final list = IndexedList(
          ['Mars', 'Alchemist', 'Pangolier', 'Snapfire', 'Oracle', 'Tiny']);
      list.add('Tide hunter');
      expect(list.indexOf('Tide hunter'), 6);
      expect(list.contains('Tide hunter'), true);
    });
    test('remove', () {
      final list = IndexedList(
          ['Mars', 'Alchemist', 'Pangolier', 'Snapfire', 'Oracle', 'Tiny']);
      list.remove('Pangolier');
      expect(list.indexOf('Pangolier'), -1);
      expect(list.indexOf('Alchemist'), 1);
      expect(list.indexOf('Snapfire'), 2);
    });
    test('setLength_chop', () {
      final list = IndexedList(
          ['Mars', 'Alchemist', 'Pangolier', 'Snapfire', 'Oracle', 'Tiny']);
      list.length = 3;
      expect(list.indexOf('Snapfire'), -1);
      expect(list.indexOf('Oracle'), -1);
      expect(list.indexOf('Tiny'), -1);
      expect(list.indexOf('Mars'), 0);
      expect(list.indexOf('Alchemist'), 1);
      expect(list.indexOf('Pangolier'), 2);
    });
    test('setLength_extend', () {
      final list = IndexedList<String?>(['Mars', 'Alchemist', 'Pangolier']);
      list.length = 6;
      expect(list.indexOf('Mars'), 0);
      expect(list.indexOf('Alchemist'), 1);
      expect(list.indexOf('Pangolier'), 2);
      expect(list.indicesOf(null), [3, 4, 5]);
    });
    test('removeRange', () {
      final list = IndexedList(
          ['Mars', 'Alchemist', 'Pangolier', 'Snapfire', 'Oracle', 'Tiny']);
      list.removeRange(2, 4);
      expect(list.indexOf('Mars'), 0);
      expect(list.indexOf('Alchemist'), 1);
      expect(list.indexOf('Oracle'), 2);
      expect(list.indexOf('Tiny'), 3);
      expect(list.indexOf('Pangolier'), -1);
      expect(list.indexOf('Snapfire'), -1);
    });
    test('setRange', () {
      final list = IndexedList(
          ['Mars', 'Alchemist', 'Pangolier', 'Snapfire', 'Oracle', 'Tiny']);
      list.setRange(2, 4, ['Lion', 'Lina']);
      expect(list.indexOf('Mars'), 0);
      expect(list.indexOf('Alchemist'), 1);
      expect(list.indexOf('Lion'), 2);
      expect(list.indexOf('Lina'), 3);
      expect(list.indexOf('Oracle'), 4);
      expect(list.indexOf('Tiny'), 5);
      expect(list.indexOf('Pangolier'), -1);
      expect(list.indexOf('Snapfire'), -1);
    });
  });
}
